# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager osdk-specific functionality.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This subpackage includes necessary modules for manipulating the operator-sdk
installation itself.
"""
