# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager module.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.
"""

from osdk_manager.cli import cli

cli()
